from django.core.management.base import BaseCommand


class DittoBaseCommand(BaseCommand):
    """
    Should probably be used as a parent by all other management commands,
    assuming they follow the usual pattern.

    Provides an output_results() method for our usual format of results.
    """

    # What we're fetching:
    singular_noun = "Thing"
    plural_noun = "Things"

    def add_arguments(self, parser):
        "We may add stuff for handling verbosity here."
        pass

    def output_results(self, results, verbosity=1):
        """results should be a list of dicts.

        Each dict is for one account.
        Each dict will look like either:

          { 'account': 'Phil Gyford',
            'success': True,
            'fetched': 200, # The number of things fetched
          }

        or:

          { 'account': 'Phil Gyford',
            'success': False,
            'messages': ['There was an error fetching data because blah'],
          }

        Or, it might not have an 'account'.
        """
        if verbosity == 0:
            return

        for result in results:
            prefix = "%s: " % result["account"] if "account" in result else ""
            if "fetched" in result:
                noun = (
                    self.singular_noun if result["fetched"] == 1 else self.plural_noun
                )
                self.stdout.write("%sFetched %s %s" % (prefix, result["fetched"], noun))

            if result["success"] is False:
                self.stderr.write(
                    "%sFailed to fetch %s: %s"
                    % (
                        prefix,
                        self.plural_noun,
                        self.format_messages(result["messages"]),
                    )
                )

    def format_messages(self, messages):
        if len(messages) == 1:
            return messages[0]
        else:
            # On separate lines, and start a newline first.
            return "%s%s" % ("\n", "\n".join(messages))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0009_account_last_fetch_id"),
    ]

    operations = [
        migrations.RemoveField(model_name="tweet", name="text",),
    ]



from django.core.management.base import BaseCommand, CommandError

from ...fetch.fetchers import UserIdFetcher, UserFetcher
from ...models import Account, User


class Command(BaseCommand):
    """For fetching data about an Account's Flickr user.

    Should create/update them in our DB, and associate the User with the Account.

        ./manage.py fetch_flickr_account_user --id=1
    """

    help = "Fetches data for an Account's Flickr User"

    def add_arguments(self, parser):
        parser.add_argument(
            "--id",
            action="store",
            default=False,
            help="The ID of the Account in the Django database.",
            type=int,
        )

    def handle(self, *args, **options):
        if options["id"] is False:
            raise CommandError("Specify an Account ID like --id=1")

        # First we need the Account object we're fetching for.
        account = False
        try:
            account = Account.objects.get(id=options["id"])
        except Account.DoesNotExist:
            self.stderr.write("No Account found with an id of '%s'" % options["id"])

        if account:
            # Then get the ID of the Flicker user for this Account's API creds.
            id_result = UserIdFetcher(account=account).fetch()

            if (
                "success" in id_result
                and id_result["success"] is True
                and "id" in id_result
            ):
                # We've got a Flickr ID, so we can get and save the user data.
                result = UserFetcher(account=account).fetch(nsid=id_result["id"])
                if "success" in result and result["success"] is True:
                    # Now we'll associate this User with the Account:
                    user = User.objects.get(nsid="35034346050@N01")
                    account.user = user
                    account.save()
                    if options.get("verbosity", 1) > 0:
                        self.stdout.write(
                            "Fetched and saved user '%s'" % result["user"]["name"]
                        )
                else:
                    if options.get("verbosity", 1) > 0:
                        self.stderr.write(
                            "Failed to fetch a user using Flickr ID '%s': %s"
                            % (id_result["id"], result["messages"][0])
                        )
            else:
                if options.get("verbosity", 1) > 0:
                    self.stderr.write(
                        "Failed to fetch a Flickr ID for this Account: %s"
                        % id_result["messages"][0]
                    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0019_auto_20151028_1556"),
    ]

    operations = [
        migrations.RenameField(
            model_name="photo", old_name="media_url", new_name="url",
        ),
        migrations.RemoveField(model_name="photo", name="page_url",),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion
import ditto.core.models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0002_auto_20150729_1704"),
    ]

    operations = [
        migrations.CreateModel(
            name="Tweet",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "time_created",
                    models.DateTimeField(
                        help_text=b"The time this item was created in the database.",
                        auto_now_add=True,
                    ),
                ),
                (
                    "time_modified",
                    models.DateTimeField(
                        help_text=b"The time this item was last saved to the database.",
                        auto_now=True,
                    ),
                ),
                ("title", models.CharField(max_length=255, blank=True)),
                (
                    "permalink",
                    models.URLField(
                        help_text=b"URL of the item on the service's website.",
                        blank=True,
                    ),
                ),
                (
                    "summary",
                    models.CharField(
                        help_text=b"eg, Initial text of a blog post, start of the description of a photo, all of a Tweet's text, etc. No HTML.",  # noqa: E501
                        max_length=255,
                        blank=True,
                    ),
                ),
                (
                    "is_private",
                    models.BooleanField(
                        default=False,
                        help_text=b"If set, this item will not be shown on public-facing pages.",  # noqa: E501
                    ),
                ),
                (
                    "fetch_time",
                    models.DateTimeField(
                        help_text=b"The time the item's data was last fetched, and was new or changed.",  # noqa: E501
                        null=True,
                        blank=True,
                    ),
                ),
                (
                    "latitude",
                    models.DecimalField(
                        null=True, max_digits=12, decimal_places=9, blank=True
                    ),
                ),
                (
                    "longitude",
                    models.DecimalField(
                        null=True, max_digits=12, decimal_places=9, blank=True
                    ),
                ),
                (
                    "raw",
                    models.TextField(
                        help_text=b"eg, the raw JSON from the API.", blank=True
                    ),
                ),
                ("text", models.CharField(max_length=255)),
                ("twitter_id", models.BigIntegerField(unique=True)),
                ("twitter_id_str", models.CharField(unique=True, max_length=20)),
                (
                    "created_at",
                    models.DateTimeField(
                        help_text=b"UTC time when this Tweet was created on Twitter"
                    ),
                ),
                (
                    "favorite_count",
                    models.PositiveIntegerField(
                        help_text=b"Approximately how many times this has been favorited"  # noqa: E501
                    ),
                ),
                (
                    "retweet_count",
                    models.PositiveIntegerField(
                        help_text=b"Number of times this has been retweeted"
                    ),
                ),
                (
                    "in_reply_to_screen_name",
                    models.CharField(
                        help_text=b"Screen name of the original Tweet's author, if this is a reply",  # noqa: E501
                        max_length=20,
                        blank=True,
                    ),
                ),
                (
                    "in_reply_to_status_id",
                    models.BigIntegerField(
                        help_text=b"The ID of the Tweet replied to, if any",
                        null=True,
                        blank=True,
                    ),
                ),
                (
                    "in_reply_to_status_id_str",
                    models.CharField(max_length=20, blank=True),
                ),
                (
                    "in_reply_to_user_id",
                    models.BigIntegerField(
                        help_text=b"ID of the original Tweet's author, if this is a reply",  # noqa: E501
                        null=True,
                        blank=True,
                    ),
                ),
                (
                    "in_reply_to_user_id_str",
                    models.CharField(max_length=20, blank=True),
                ),
                (
                    "language",
                    models.CharField(
                        default=b"und",
                        help_text=b"A BCP 47 language identifier, or 'und' if it couldn't be detected",  # noqa: E501
                        max_length=20,
                    ),
                ),
                (
                    "place_attribute_street_address",
                    models.CharField(max_length=255, blank=True),
                ),
                ("place_full_name", models.CharField(max_length=255, blank=True)),
                ("place_country", models.CharField(max_length=255, blank=True)),
                (
                    "quoted_status_id",
                    models.BigIntegerField(
                        help_text=b"The ID of the Tweet quoted, if any",
                        null=True,
                        blank=True,
                    ),
                ),
                ("quoted_status_id_str", models.CharField(max_length=20, blank=True)),
                (
                    "source",
                    models.CharField(
                        help_text=b"Utility used to post the Tweet",
                        max_length=255,
                        blank=True,
                    ),
                ),
            ],
            options={"ordering": ["-created_at"]},
            bases=(ditto.core.models.DiffModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="User",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "time_created",
                    models.DateTimeField(
                        help_text=b"The time this item was created in the database.",
                        auto_now_add=True,
                    ),
                ),
                (
                    "time_modified",
                    models.DateTimeField(
                        help_text=b"The time this item was last saved to the database.",
                        auto_now=True,
                    ),
                ),
                ("twitter_id", models.BigIntegerField(unique=True)),
                ("twitter_id_str", models.CharField(unique=True, max_length=20)),
                (
                    "screen_name",
                    models.CharField(
                        help_text=b"Username, eg, 'samuelpepys'", max_length=20
                    ),
                ),
                (
                    "name",
                    models.CharField(help_text=b"eg, 'Samuel Pepys'", max_length=30),
                ),
                (
                    "url",
                    models.URLField(
                        help_text=b"A URL provided by the user as part of their profile",  # noqa: E501
                        blank=True,
                    ),
                ),
                (
                    "is_private",
                    models.BooleanField(
                        default=False,
                        help_text=b"True if this user is 'protected' or private",
                    ),
                ),
                ("is_verified", models.BooleanField(default=False)),
                (
                    "created_at",
                    models.DateTimeField(
                        help_text=b"UTC time when this account was created on Twitter"
                    ),
                ),
                ("description", models.CharField(max_length=255, blank=True)),
                ("location", models.CharField(max_length=255, blank=True)),
                ("time_zone", models.CharField(max_length=255, blank=True)),
                ("profile_image_url", models.URLField(max_length=255, blank=True)),
                (
                    "profile_image_url_https",
                    models.URLField(max_length=255, blank=True),
                ),
                (
                    "favorites_count",
                    models.PositiveIntegerField(
                        help_text=b"The number of tweets this user has favorited in the account\xe2\x80\x99s lifetime"  # noqa: E501
                    ),
                ),
                (
                    "followers_count",
                    models.PositiveIntegerField(
                        help_text=b"The number of followers this account has"
                    ),
                ),
                (
                    "friends_count",
                    models.PositiveIntegerField(
                        help_text=b"Tne number of users this account is following."
                    ),
                ),
                (
                    "listed_count",
                    models.PositiveIntegerField(
                        help_text=b"The number of public lists this user is a member of"
                    ),
                ),
                (
                    "statuses_count",
                    models.PositiveIntegerField(
                        help_text=b"The number of tweets, including retweets, by this user"  # noqa: E501
                    ),
                ),
                (
                    "fetch_time",
                    models.DateTimeField(
                        help_text=b"The time the data was last fetched, and was new or changed.",  # noqa: E501
                        null=True,
                        blank=True,
                    ),
                ),
                (
                    "raw",
                    models.TextField(
                        help_text=b"eg, the raw JSON from the API.", blank=True
                    ),
                ),
            ],
            options={"ordering": ["screen_name"]},
            bases=(ditto.core.models.DiffModelMixin, models.Model),
        ),
        migrations.AlterField(
            model_name="account",
            name="screen_name",
            field=models.CharField(
                help_text=b"Username, eg, 'samuelpepys'",
                unique=True,
                max_length=20,
                blank=True,
            ),
        ),
        migrations.AddField(
            model_name="tweet",
            name="user",
            field=models.ForeignKey(to="twitter.User", on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name="account",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.SET_NULL,
                blank=True,
                to="twitter.User",
                null=True,
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-12 10:12
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("lastfm", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="scrobble",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="scrobbles",
                to="lastfm.Account",
            ),
        ),
    ]



import factory

from . import models
from ..core.utils import datetime_now


class AccountFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Account

    username = factory.Sequence(lambda n: "user%04d" % n)
    realname = factory.Sequence(lambda n: "Real Name %04d" % n)
    api_key = factory.Sequence(lambda n: "api-key-%04d" % n)


class ArtistFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Artist

    name = factory.Sequence(lambda n: "Artist %04d" % n)
    slug = factory.Sequence(lambda n: "artist+%04d" % n)
    original_slug = factory.Sequence(lambda n: "Artist+%04d" % n)


class TrackFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Track

    name = factory.Sequence(lambda n: "Track %04d" % n)
    slug = factory.Sequence(lambda n: "track+%04d" % n)
    original_slug = factory.Sequence(lambda n: "Track+%04d" % n)
    artist = factory.SubFactory(ArtistFactory)


class AlbumFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Album

    name = factory.Sequence(lambda n: "Album %04d" % n)
    slug = factory.Sequence(lambda n: "album+%04d" % n)
    original_slug = factory.Sequence(lambda n: "Album+%04d" % n)
    artist = factory.SubFactory(ArtistFactory)


class ScrobbleFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Scrobble

    account = factory.SubFactory(AccountFactory)
    track = factory.SubFactory(TrackFactory)
    artist = factory.SubFactory(ArtistFactory)
    post_time = datetime_now()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-11-17 16:22
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("lastfm", "0004_auto_20161026_1540"),
    ]

    operations = [
        migrations.AlterField(
            model_name="scrobble",
            name="post_time",
            field=models.DateTimeField(
                blank=True,
                db_index=True,
                help_text="The time the item was originally posted/created on its service.",  # noqa: E501
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="scrobble",
            name="summary",
            field=models.CharField(
                blank=True,
                help_text="eg, Brief summary or excerpt of item's text content. No linebreaks or HTML.",  # noqa: E501
                max_length=255,
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-26 15:39
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("lastfm", "0002_auto_20161012_1012"),
    ]

    operations = [
        migrations.AddField(
            model_name="album",
            name="original_slug",
            field=models.TextField(
                blank=True, help_text="As used on Last.fm. Mixed case.", null=True
            ),
        ),
        migrations.AddField(
            model_name="artist",
            name="original_slug",
            field=models.TextField(
                blank=True, help_text="As used on Last.fm. Mixed case.", null=True
            ),
        ),
        migrations.AddField(
            model_name="track",
            name="original_slug",
            field=models.TextField(
                blank=True, help_text="As used on Last.fm. Mixed case.", null=True
            ),
        ),
        migrations.AlterField(
            model_name="album",
            name="slug",
            field=models.TextField(db_index=True, help_text="Lowercase"),
        ),
        migrations.AlterField(
            model_name="artist",
            name="slug",
            field=models.TextField(help_text="Lowercase", unique=True),
        ),
        migrations.AlterField(
            model_name="track",
            name="slug",
            field=models.TextField(db_index=True, help_text="Lowercase"),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0027_remove_user_profile_image_url"),
    ]

    operations = [
        migrations.RenameField(
            model_name="user", old_name="favorites_count", new_name="favourites_count",
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-13 16:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0043_remove_media_is_private"),
    ]

    operations = [
        migrations.AlterField(
            model_name="tweet", name="text", field=models.TextField(),
        ),
    ]



# coding: utf-8
from django.db import models
from django.forms.models import model_to_dict

from .managers import PublicItemManager
from .utils import truncate_string


class TimeStampedModelMixin(models.Model):
    "Should be mixed in to all models."
    time_created = models.DateTimeField(
        auto_now_add=True, help_text="The time this item was created in the database."
    )
    time_modified = models.DateTimeField(
        auto_now=True, help_text="The time this item was last saved to the database."
    )

    class Meta:
        abstract = True


class DiffModelMixin(object):
    """A model mixin that tracks model fields' values and provide some useful
    api to know what fields have been changed.

    eg:
    Get an object from the database.
    Set some of its properties.
    Call `myObj.has_changed` to see if any fields are different to in the DB.

    From http://stackoverflow.com/a/13842223/250962
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__initial = self._dict

    @property
    def diff(self):
        """Returns a dict of properties that have changed, with values a list
        of before/after changes. eg:
        `{'categories': (None, [1, 3, 5]), 'rank': (0, 42)}`
        """
        d1 = self.__initial
        d2 = self._dict
        diffs = [(k, (v, d2[k])) for k, v in d1.items() if v != d2[k]]
        return dict(diffs)

    @property
    def has_changed(self):
        "Returns True if any of the properties have changed."
        return bool(self.diff)

    @property
    def changed_fields(self):
        "Returns a list of property names for any that have changed."
        return self.diff.keys()

    def get_field_diff(self, field_name):
        "Returns a diff for field if it's changed and None otherwise."
        return self.diff.get(field_name, None)

    def save(self, *args, **kwargs):
        "Saves model and set initial state."
        super().save(*args, **kwargs)
        self.__initial = self._dict

    @property
    def _dict(self):
        return model_to_dict(self, fields=[field.name for field in self._meta.fields])


class DittoItemModel(TimeStampedModelMixin, DiffModelMixin, models.Model):
    """
    A content item on whatever service we're copying.
    eg, a Tweet, a Photo, a Bookmark, etc.

    Should be inherited by a model in child apps.
    eg, Tweet, Photo, Bookmark, etc.
    """

    # Should be overridden for child classes.
    # eg, 'flickr_photo', 'twitter_tweet', etc.
    # Used in templates.
    ditto_item_name = "set__ditto_item_name__in_child_class"

    title = models.CharField(blank=True, max_length=255)
    permalink = models.URLField(
        blank=True, help_text="URL of the item on the service's website."
    )
    # Ensures that all children have a common short piece of text for display:
    summary = models.CharField(
        blank=True,
        max_length=255,
        help_text=(
            "eg, Brief summary or excerpt of item's text content. "
            "No linebreaks or HTML."
        ),
    )
    is_private = models.BooleanField(
        default=False,
        help_text="If true, this item will not be shown on public-facing pages.",
    )
    fetch_time = models.DateTimeField(
        null=True, blank=True, help_text="The time the item's data was last fetched."
    )

    post_time = models.DateTimeField(
        null=True,
        blank=True,
        db_index=True,
        help_text="The time the item was originally posted/created on its service.",
    )
    post_year = models.PositiveSmallIntegerField(
        null=True, blank=True, db_index=True, help_text="Set automatically on save"
    )

    # Obviously not relevant to some items, like Bookmarks.
    latitude = models.DecimalField(
        null=True, blank=True, max_digits=9, decimal_places=6
    )
    longitude = models.DecimalField(
        null=True, blank=True, max_digits=9, decimal_places=6
    )

    raw = models.TextField(blank=True, help_text="eg, the raw JSON from the API.")

    # All Items (eg, used in Admin):
    objects = models.Manager()

    # All Items which aren't private. Should ALWAYS be used for public pages:
    public_objects = PublicItemManager()

    class Meta:
        abstract = True
        get_latest_by = "post_time"

    def __str__(self):
        return self.title

    def save(self, *args, **kwargs):
        self.summary = self._make_summary()
        if self.post_time:
            self.post_year = self.post_time.year
        else:
            self.post_year = None
        super().save(*args, **kwargs)

    def _summary_source(self):
        """
        Child classes can return a string that's used to make the truncated,
        HTML-free `summary`.
        e.g.:
            return self.description
        """
        return ""

    def _make_summary(self):
        """
        Returns the string to be used for the `summary` property.
        """
        return (
            truncate_string(
                self._summary_source(),
                strip_html=True,
                chars=255,
                truncate="…",
                at_word_boundary=True,
            )
            .replace("\n", " ")
            .replace("\r", " ")
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("pinboard", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(model_name="bookmark", name="shared",),
        migrations.AlterField(
            model_name="bookmark",
            name="raw",
            field=models.TextField(
                help_text=b"eg, the raw JSON from the API.", blank=True
            ),
        ),
    ]



from django.core.management.base import CommandError

from ....core.management.commands import DittoBaseCommand
from ...fetch import ScrobblesMultiAccountFetcher


class Command(DittoBaseCommand):

    # What we're fetching:
    singular_noun = "Scrobble"
    plural_noun = "Scrobbles"

    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument(
            "--account",
            action="store",
            default=False,
            help='The username of the Last.fm user to fetch Scrobbles for. e.g. "rj".',
        )

        parser.add_argument(
            "--days",
            action="store",
            default=False,
            help=(
                'Number of days to fetch. Or "all" for ALL scrobbles. '
                "Or omit to only fetch since the most recently fetched scrobble."
            ),
        )

    def handle(self, *args, **options):
        # We might be fetching for a specific account or all (None).
        username = options["account"] if options["account"] else None

        # Default:
        fetch_type = "recent"

        if options["days"]:
            # Will be either 'all' or a number; make the number an int.
            if options["days"].isdigit():
                options["days"] = int(options["days"])
                fetch_type = "days"

            elif options["days"] == "all":
                fetch_type = "all"

            else:
                raise CommandError("--days should be an integer or 'all'.")

        fetcher = ScrobblesMultiAccountFetcher(username=username)

        if fetch_type == "days":
            results = fetcher.fetch(fetch_type=fetch_type, days=options["days"])
        else:
            results = fetcher.fetch(fetch_type=fetch_type)

        self.output_results(results, options.get("verbosity", 1))



from django.conf.urls import url

from . import views


app_name = "twitter"

urlpatterns = [
    url(regex=r"^$", view=views.HomeView.as_view(), name="home"),
    url(regex=r"^likes/$", view=views.FavoriteListView.as_view(), name="favorite_list"),
    url(
        regex=r"^(?P<screen_name>\w+)/$",
        view=views.UserDetailView.as_view(),
        name="user_detail",
    ),
    url(
        regex=r"^(?P<screen_name>\w+)/likes/$",
        view=views.AccountFavoriteListView.as_view(),
        name="account_favorite_list",
    ),
    url(
        regex=r"^(?P<screen_name>\w+)/(?P<twitter_id>\d+)/$",
        view=views.TweetDetailView.as_view(),
        name="tweet_detail",
    ),
]



from datetime import timedelta

from django.http import Http404
from django.utils.translation import ugettext as _
from django.views.generic import DetailView, TemplateView
from django.views.generic.detail import SingleObjectMixin

from ..core.utils import datetime_now
from ..core.views import PaginatedListView
from .models import Account, Album, Artist, Scrobble, Track


class AccountsMixin(object):
    """
    View Mixin for adding an `account_list` to context, with all Accounts in.
    And the total counts of Scrobbles, Albums, Artists and Tracks.
    """

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["account_list"] = Account.objects.all()
        context["counts"] = {
            "albums": Album.objects.all().count(),
            "artists": Artist.objects.all().count(),
            "scrobbles": Scrobble.objects.all().count(),
            "tracks": Track.objects.all().count(),
        }
        return context


class HomeView(AccountsMixin, TemplateView):
    "Uses template tags to display charts, recent Scrobbles, etc."
    template_name = "lastfm/home.html"


class ScrobbleListView(AccountsMixin, PaginatedListView):
    "A multi-page list of Scrobbles, most recent first."
    template_name = "lastfm/scrobble_list.html"
    model = Scrobble

    def get_queryset(self):
        "Pre-fetch Artists and Tracks to reduce number of queries."
        qs = super(ScrobbleListView, self).get_queryset()
        return qs.prefetch_related("artist", "track")


class ChartPaginatedListView(PaginatedListView):
    """
    For the Album, Artist and Track ListViews that are charts ordered by
    scrobble_count.
    """

    ordering = "-scrobble_count"

    def get_queryset(self):
        return self.get_queryset_with_counts()

    def get_queryset_with_counts(self):
        """
        Includes scrobble_counts with returned objects, and adds ability to
        limit results to a time period based on the supplied `days`.

        This was just get_queryset(), but then combining this class with the
        SingleAccountMixin didn't work; child classes were using
        SingleObjectMixin's get_queryset().

        So now, each child class has to call this method from its own
        get_queryset().
        """
        queryset = self.model._default_manager

        # This will contain filters on the queryset:
        qs_kwargs = {}

        days = self.get_days()  # eg 7, 30 or 'all'

        if days != "all":
            time_ago = datetime_now() - timedelta(days=days)
            qs_kwargs["min_post_time"] = time_ago

        if hasattr(self, "object") and isinstance(self.object, Account):
            # We need to filter the results to only get Tracks (or whatever)
            # scrobbled by this Account.
            qs_kwargs["account"] = self.object

        queryset = queryset.with_scrobble_counts(**qs_kwargs)

        ordering = (self.get_ordering(),)
        queryset = queryset.order_by(*ordering)

        return queryset

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["current_days"] = str(self.get_days())
        context["valid_days"] = self.get_valid_days()
        return context

    def get_valid_days(self):
        return [
            "7",
            "30",
            "90",
            "180",
            "365",
            "all",
        ]

    def get_days(self):
        """
        Returns the current number of recent days we're viewing.

        Returns an integer (number of days) or 'all'.
        """
        days = self.request.GET.get("days")
        if days and days in self.get_valid_days():
            try:
                return int(days)
            except ValueError:
                return days
        else:
            return "all"


class AlbumListView(AccountsMixin, ChartPaginatedListView):
    "A multi-page chart of most-scrobbled Tracks."
    template_name = "lastfm/album_list.html"
    model = Album


class ArtistListView(AccountsMixin, ChartPaginatedListView):
    "A multi-page chart of most-scrobbled Tracks."
    template_name = "lastfm/artist_list.html"
    model = Artist


class TrackListView(AccountsMixin, ChartPaginatedListView):
    "A multi-page chart of most-scrobbled Tracks."
    template_name = "lastfm/track_list.html"
    model = Track


class AlbumDetailView(DetailView):
    "A single Album by a particular Artist."
    model = Album

    def get_object(self, queryset=None):
        """
        Returns the object the view is displaying.

        This is based on DetailView's method.
        """
        # Use a custom queryset if provided; this is required for subclasses
        # like DateDetailView
        if queryset is None:
            queryset = self.get_queryset()

        artist_slug = self.kwargs.get("artist_slug")
        album_slug = self.kwargs.get("album_slug")

        if artist_slug is None or album_slug is None:
            raise AttributeError(
                "AlbumDetailView must be called with " "artist_slug and album_slug"
            )

        artist = Artist.objects.get(slug=artist_slug)
        queryset = queryset.filter(artist=artist, slug=album_slug)

        try:
            # Get the single item from the filtered queryset
            obj = queryset.get()
        except queryset.model.DoesNotExist:
            raise Http404(
                _("No %(verbose_name)s found matching the query")
                % {"verbose_name": queryset.model._meta.verbose_name}
            )
        return obj


class ArtistDetailView(DetailView):
    "One Artist. Uses a template tag to display a chart of their Tracks."
    model = Artist
    slug_url_kwarg = "artist_slug"


class ArtistAlbumsView(DetailView):
    "One Artist. Uses a template tag to display a chart of their Albums."
    model = Artist
    slug_url_kwarg = "artist_slug"
    template_name = "lastfm/artist_albums.html"


class TrackDetailView(DetailView):
    "One Track by a particular Artist."
    model = Track

    def get_object(self, queryset=None):
        """
        Returns the object the view is displaying.

        This is based on DetailView's method.
        """
        # Use a custom queryset if provided; this is required for subclasses
        # like DateDetailView
        if queryset is None:
            queryset = self.get_queryset()

        artist_slug = self.kwargs.get("artist_slug")
        track_slug = self.kwargs.get("track_slug")

        if artist_slug is None or track_slug is None:
            raise AttributeError(
                "TrackDetailView must be called with " "artist_slug and track_slug"
            )

        artist = Artist.objects.get(slug=artist_slug)
        queryset = queryset.filter(artist=artist, slug=track_slug)

        try:
            # Get the single item from the filtered queryset
            obj = queryset.get()
        except queryset.model.DoesNotExist:
            raise Http404(
                _("No %(verbose_name)s found matching the query")
                % {"verbose_name": queryset.model._meta.verbose_name}
            )
        return obj


class SingleAccountMixin(SingleObjectMixin):
    """Used for views that need data about an Account based on username in
    the URL.
    """

    slug_field = "username"
    slug_url_kwarg = "username"

    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=Account.objects.all())
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        "Add counts of things for this Account only."
        context = super().get_context_data(**kwargs)

        qs = Scrobble.objects.filter(account=self.object)

        context["counts"] = {
            "albums": qs.exclude(album_id__isnull=True)
            .values("album_id")
            .distinct()
            .count(),
            "artists": qs.values("artist_id").distinct().count(),
            "scrobbles": qs.count(),
            "tracks": qs.values("track_id").distinct().count(),
        }
        return context


class UserDetailView(SingleAccountMixin, DetailView):
    "Overview of the user; top 10s of everything."
    template_name = "lastfm/user_detail.html"
    model = Account


class UserAlbumListView(SingleAccountMixin, ChartPaginatedListView):
    "Chart of Albums scrobbled by one user."
    template_name = "lastfm/user_album_list.html"
    model = Album

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["album_list"] = context["object_list"]
        return context

    def get_queryset(self):
        return self.get_queryset_with_counts()


class UserArtistListView(SingleAccountMixin, ChartPaginatedListView):
    "Chart of Artists scrobbled by one user."
    template_name = "lastfm/user_artist_list.html"
    model = Artist

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["artist_list"] = context["object_list"]
        return context

    def get_queryset(self):
        return self.get_queryset_with_counts()


class UserScrobbleListView(SingleAccountMixin, PaginatedListView):
    "All scrobbles by one user."
    template_name = "lastfm/user_scrobble_list.html"
    model = Scrobble

    def get_queryset(self):
        """
        All Scrobbles by this Account.
        And pre-fetch Artists and Tracks to reduce number of queries.
        """
        queryset = super().get_queryset()
        return queryset.filter(account=self.object).prefetch_related("artist", "track")

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["scrobble_list"] = context["object_list"]
        return context


class UserTrackListView(SingleAccountMixin, ChartPaginatedListView):
    "Chart of Tracks scrobbled by one user."
    template_name = "lastfm/user_track_list.html"
    model = Track

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["track_list"] = context["object_list"]
        return context

    def get_queryset(self):
        return self.get_queryset_with_counts()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0025_auto_20151109_1651"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="created_at",
            field=models.DateTimeField(
                help_text="UTC time when this account was created on Twitter",
                blank=True,
                null=True,
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0005_auto_20160414_1427"),
    ]

    operations = [
        migrations.RenameField(
            model_name="photo",
            old_name="large_2400_height",
            new_name="large_2048_height",
        ),
        migrations.RenameField(
            model_name="photo",
            old_name="large_2400_width",
            new_name="large_2048_width",
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0004_auto_20160414_1120"),
    ]

    operations = [
        migrations.RenameField(
            model_name="photo", old_name="height_mp4_hd", new_name="hd_mp4_height",
        ),
        migrations.RenameField(
            model_name="photo", old_name="width_mp4_hd", new_name="hd_mp4_width",
        ),
        migrations.RenameField(
            model_name="photo",
            old_name="height_mp4_mobile",
            new_name="mobile_mp4_height",
        ),
        migrations.RenameField(
            model_name="photo",
            old_name="width_mp4_mobile",
            new_name="mobile_mp4_width",
        ),
        migrations.RenameField(
            model_name="photo", old_name="height_mp4_site", new_name="site_mp4_height",
        ),
        migrations.RenameField(
            model_name="photo", old_name="width_mp4_site", new_name="site_mp4_width",
        ),
        migrations.RemoveField(model_name="photo", name="height",),
        migrations.RemoveField(model_name="photo", name="height_b",),
        migrations.RemoveField(model_name="photo", name="height_c",),
        migrations.RemoveField(model_name="photo", name="height_h",),
        migrations.RemoveField(model_name="photo", name="height_k",),
        migrations.RemoveField(model_name="photo", name="height_m",),
        migrations.RemoveField(model_name="photo", name="height_n",),
        migrations.RemoveField(model_name="photo", name="height_o",),
        migrations.RemoveField(model_name="photo", name="height_t",),
        migrations.RemoveField(model_name="photo", name="height_video_original",),
        migrations.RemoveField(model_name="photo", name="height_z",),
        migrations.RemoveField(model_name="photo", name="width",),
        migrations.RemoveField(model_name="photo", name="width_b",),
        migrations.RemoveField(model_name="photo", name="width_c",),
        migrations.RemoveField(model_name="photo", name="width_h",),
        migrations.RemoveField(model_name="photo", name="width_k",),
        migrations.RemoveField(model_name="photo", name="width_m",),
        migrations.RemoveField(model_name="photo", name="width_n",),
        migrations.RemoveField(model_name="photo", name="width_o",),
        migrations.RemoveField(model_name="photo", name="width_t",),
        migrations.RemoveField(model_name="photo", name="width_video_original",),
        migrations.RemoveField(model_name="photo", name="width_z",),
        migrations.AddField(
            model_name="photo",
            name="large_1600_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="large_1600_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="large_2400_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="large_2400_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="large_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="large_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="medium_640_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="medium_640_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="medium_800_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="medium_800_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="medium_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="medium_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="original_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="original_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="small_320_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="small_320_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="small_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="small_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="thumbnail_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="thumbnail_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="video_original_height",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="photo",
            name="video_original_width",
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0032_auto_20160414_1637"),
    ]

    operations = [
        migrations.AlterField(
            model_name="media",
            name="aspect_ratio",
            field=models.CharField(
                help_text='eg, "4:3" or "16:9"', max_length=5, blank=True
            ),
        ),
        migrations.AlterField(
            model_name="media",
            name="dash_url",
            field=models.URLField(verbose_name="MPEG-DASH URL", default="", blank=True),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="media",
            name="mp4_bitrate_1",
            field=models.PositiveIntegerField(
                help_text="Lowest bitrate",
                null=True,
                verbose_name="MP4 Bitrate (1)",
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name="media",
            name="mp4_bitrate_2",
            field=models.PositiveIntegerField(
                help_text="Medium bitrate",
                null=True,
                verbose_name="MP4 Bitrate (2)",
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name="media",
            name="mp4_bitrate_3",
            field=models.PositiveIntegerField(
                help_text="Highest bitrate",
                null=True,
                verbose_name="MP4 Bitrate (3)",
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name="media",
            name="mp4_url_1",
            field=models.URLField(
                help_text="Lowest bitrate",
                verbose_name="MP4 URL (1)",
                default="",
                blank=True,
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="media",
            name="mp4_url_2",
            field=models.URLField(
                help_text="Medium bitrate",
                verbose_name="MP4 URL (2)",
                default="",
                blank=True,
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="media",
            name="mp4_url_3",
            field=models.URLField(
                help_text="Highest bitrate",
                verbose_name="MP4 URL (3)",
                default="",
                blank=True,
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="media",
            name="webm_url",
            field=models.URLField(verbose_name="WebM URL", default="", blank=True),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="media",
            name="xmpeg_url",
            field=models.URLField(verbose_name="X-MPEG URL", default="", blank=True),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("pinboard", "0002_auto_20150626_1521"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="bookmark", unique_together=set([("account", "url")]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-03 14:57
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0011_auto_20160502_1645"),
    ]

    operations = [
        migrations.AlterField(
            model_name="photoset",
            name="flickr_id",
            field=models.BigIntegerField(db_index=True, unique=True),
        ),
        migrations.AlterField(
            model_name="photoset",
            name="photo_count",
            field=models.PositiveIntegerField(
                default=0, help_text="The number of photos in the set on Flickr"
            ),
        ),
        migrations.AlterField(
            model_name="photoset",
            name="video_count",
            field=models.PositiveIntegerField(
                default=0, help_text="The number of videos in the set on Flickr"
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="nsid",
            field=models.CharField(
                db_index=True, max_length=50, unique=True, verbose_name="NSID"
            ),
        ),
    ]



from django.conf.urls import url

from . import views


app_name = "pinboard"

urlpatterns = [
    url(regex=r"^$", view=views.HomeView.as_view(), name="home"),
    url(regex=r"^to-read/$", view=views.ToreadListView.as_view(), name="toread"),
    url(regex=r"^tags/$", view=views.TagListView.as_view(), name="tag_list"),
    url(
        regex=r"^tags/(?P<slug>[^/]+)/$",
        view=views.TagDetailView.as_view(),
        name="tag_detail",
    ),
    url(
        regex=r"^(?P<username>\w+)/$",
        view=views.AccountDetailView.as_view(),
        name="account_detail",
    ),
    url(
        regex=r"^(?P<username>\w+)/to-read/$",
        view=views.AccountToreadView.as_view(),
        name="account_toread",
    ),
    url(
        regex=r"^(?P<username>\w+)/tags/(?P<tag_slug>[^/]+)/$",
        view=views.AccountTagDetailView.as_view(),
        name="account_tag_detail",
    ),
    url(
        regex=r"^(?P<username>\w+)/(?P<hash>\w+)/$",
        view=views.BookmarkDetailView.as_view(),
        name="bookmark_detail",
    ),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-29 16:05
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0008_auto_20160429_1559"),
    ]

    operations = [
        migrations.AddField(
            model_name="photoset",
            name="fetch_time",
            field=models.DateTimeField(
                blank=True,
                help_text="The time the item's data was last fetched.",
                null=True,
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("pinboard", "0004_auto_20150626_1603"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bookmark",
            name="summary",
            field=models.CharField(
                help_text=b"eg, Initial text of a blog post, start of the description of a photo, all of a Tweet's text, etc.",  # noqa: E501
                max_length=255,
                blank=True,
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0016_auto_20151002_1525"),
    ]

    operations = [
        migrations.AddField(
            model_name="account",
            name="is_active",
            field=models.BooleanField(
                help_text="If false, new Tweets won't be fetched.", default=True
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Account",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "time_created",
                    models.DateTimeField(
                        help_text=b"The time this item was created in the database.",
                        auto_now_add=True,
                    ),
                ),
                (
                    "time_modified",
                    models.DateTimeField(
                        help_text=b"The time this item was last saved to the database.",
                        auto_now=True,
                    ),
                ),
                ("screen_name", models.CharField(max_length=20, blank=True)),
                ("consumer_key", models.CharField(max_length=255, blank=True)),
                ("consumer_secret", models.CharField(max_length=255, blank=True)),
                ("access_token", models.CharField(max_length=255, blank=True)),
                ("access_token_secret", models.CharField(max_length=255, blank=True)),
            ],
            options={"ordering": ["screen_name"]},
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0011_tweet_text"),
    ]

    operations = [
        migrations.RenameField(
            model_name="account", old_name="last_fetch_id", new_name="last_recent_id",
        ),
        migrations.AddField(
            model_name="account",
            name="last_favorite_id",
            field=models.BigIntegerField(
                help_text=b"The Twitter ID of the most recent favorited Tweet fetched.",
                null=True,
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="is_private",
            field=models.BooleanField(
                default=False, help_text=b"True if this user is 'protected'"
            ),
        ),
    ]



from django.contrib import admin
from django.db import models
from django.forms import Textarea, TextInput
from django.utils.safestring import mark_safe

from ..core.admin import DittoItemModelAdmin
from .models import Account, Media, Tweet, User


@admin.register(Account)
class AccountAdmin(admin.ModelAdmin):
    list_display = (
        "user",
        "is_active",
        "time_created",
        "time_modified",
    )

    fieldsets = (
        (None, {"fields": ("user", "is_active",)}),
        (
            "API",
            {
                "fields": (
                    "consumer_key",
                    "consumer_secret",
                    "access_token",
                    "access_token_secret",
                ),
                "description": (
                    "Keys and secrets require creation of an app at "
                    '<a href="https://apps.twitter.com/">apps.twitter.com</a>'
                ),
            },
        ),
        (
            "Data",
            {
                "fields": (
                    "last_recent_id",
                    "last_favorite_id",
                    "time_created",
                    "time_modified",
                )
            },
        ),
    )

    formfield_overrides = {
        # Make the inputs full-width.
        models.CharField: {"widget": TextInput(attrs={"class": "vLargeTextField"})},
    }

    readonly_fields = (
        "time_created",
        "time_modified",
    )


class TweetsMediaInline(admin.TabularInline):
    model = Media.tweets.through
    extra = 0
    raw_id_fields = (
        "media",
        "tweet",
    )


@admin.register(Media)
class MediaAdmin(admin.ModelAdmin):

    list_display = (
        "show_thumb",
        "media_type",
        "time_created",
    )
    list_display_links = ("show_thumb",)
    list_filter = ("time_created",)
    date_hierarchy = "time_created"

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "show_image",
                    "media_type",
                    "twitter_id",
                    "image_url",
                    "image_file",
                    "mp4_file",
                ),
            },
        ),
        (
            "Sizes",
            {
                "classes": ("collapse",),
                "fields": (
                    "large_w",
                    "large_h",
                    "medium_w",
                    "medium_h",
                    "small_w",
                    "small_h",
                    "thumb_w",
                    "thumb_h",
                ),
            },
        ),
        (
            "Video",
            {
                "classes": ("collapse",),
                "fields": (
                    "aspect_ratio",
                    "duration",
                    "mp4_url",
                    "dash_url",
                    "xmpeg_url",
                ),
            },
        ),
        (
            "Data",
            {"classes": ("collapse",), "fields": ("time_created", "time_modified",)},
        ),
    )

    inlines = [
        TweetsMediaInline,
    ]

    readonly_fields = (
        "show_image",
        "time_created",
        "time_modified",
    )
    exclude = ("tweets",)

    def show_thumb(self, instance):
        return mark_safe(
            '<img src="%(url)s" width="%(w)s" height="%(h)s" />'
            % {
                "url": instance.thumbnail_url,
                "w": instance.thumbnail_w,
                "h": instance.thumbnail_h,
            }
        )

    show_thumb.short_description = ""

    def show_image(self, instance):
        if instance.media_type == "photo":
            html = '<img src="%(url)s" width="%(w)s" height="%(h)s" />' % {
                "url": instance.small_url,
                "w": instance.small_w,
                "h": instance.small_h,
            }
        else:
            html = (
                '<video width="%(w)s" height="%(h)s" poster="%(img)s" controls preload="metadata"><source src="%(video)s" type="%(mime)s"></video>'  # noqa: E501
                % {
                    "w": instance.small_w,
                    "h": instance.small_h,
                    "img": instance.small_url,
                    "video": instance.video_url,
                    "mime": instance.video_mime_type,
                }
            )
        return mark_safe(html)

    show_image.short_description = "Image"


@admin.register(Tweet)
class TweetAdmin(DittoItemModelAdmin):
    list_display = (
        "user",
        "is_private",
        "title",
        "post_time",
    )
    list_display_links = ("title",)
    list_filter = (
        "post_time",
        "fetch_time",
    )
    search_fields = ("text",)

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "user",
                    "text",
                    "text_html",
                    "title",
                    "summary",
                    "post_time",
                    "post_year_str",
                    "is_private",
                    "twitter_id",
                    "permalink",
                )
            },
        ),
        (
            None,
            {
                "fields": (
                    "favorite_count",
                    "retweet_count",
                    "media_count",
                    "language",
                    "source",
                    "in_reply_to_screen_name",
                    "in_reply_to_status_id",
                    "in_reply_to_user_id",
                    "quoted_status_id",
                    "retweeted_status_id",
                )
            },
        ),
        (
            "Location",
            {
                "classes": ("collapse",),
                "fields": (
                    "latitude",
                    "longitude",
                    "place_attribute_street_address",
                    "place_full_name",
                    "place_country",
                ),
            },
        ),
        (
            "Data",
            {
                "classes": ("collapse",),
                "fields": ("raw", "fetch_time", "time_created", "time_modified",),
            },
        ),
    )

    readonly_fields = (
        "post_year_str",
        "text_html",
        "raw",
        "fetch_time",
        "time_created",
        "time_modified",
    )

    inlines = [
        TweetsMediaInline,
    ]

    formfield_overrides = {
        # Make the inputs full-width.
        models.CharField: {"widget": TextInput(attrs={"class": "vLargeTextField"})},
        # Reduce the number of rows.
        models.TextField: {
            "widget": Textarea(
                attrs={"class": "vLargeTextField", "cols": 40, "rows": 5}
            )
        },
    }


@admin.register(User)
class UserAdmin(admin.ModelAdmin):
    list_display = (
        "show_avatar",
        "screen_name",
        "name",
        "is_private",
        "fetch_time",
    )
    list_display_links = (
        "show_avatar",
        "screen_name",
    )
    search_fields = (
        "screen_name",
        "name",
    )

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "screen_name",
                    "name",
                    "is_private",
                    "is_verified",
                    "profile_image_url_https",
                    "avatar",
                    "url",
                    "description",
                    "description_html",
                    "location",
                    "time_zone",
                    "twitter_id",
                    "created_at",
                )
            },
        ),
        ("Data", {"fields": ("raw", "fetch_time", "time_created", "time_modified",)}),
    )

    formfield_overrides = {
        # Make the inputs full-width.
        models.CharField: {"widget": TextInput(attrs={"class": "vLargeTextField"})},
    }

    readonly_fields = (
        "description_html",
        "raw",
        "fetch_time",
        "time_created",
        "time_modified",
    )

    def show_avatar(self, instance):
        return mark_safe(
            '<img src="%s" width="24" height="24" />' % (instance.avatar_url)
        )

    show_avatar.short_description = ""



# Generated by Django 2.0 on 2018-01-04 14:57

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0022_photo_taken_year"),
    ]

    operations = [
        migrations.AlterField(
            model_name="photoset",
            name="primary_photo",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="primary_photosets",
                to="flickr.Photo",
            ),
        ),
    ]



# coding: utf-8
from django.core.management.base import CommandError

from ....core.management.commands import DittoBaseCommand
from ...fetch import (
    AllBookmarksFetcher,
    DateBookmarksFetcher,
    RecentBookmarksFetcher,
    UrlBookmarksFetcher,
)


class Command(DittoBaseCommand):
    """Fetches bookmarks from Pinboard

    Fetch all bookmarks, from all accounts:
    ./manage.py fetch_pinboard_bookmarks --all

    Fetch bookmarks posted on one date:
    ./manage.py fetch_pinboard_bookmarks --date=2015-06-20

    Fetch the 20 most recent bookmarks:
    ./manage.py fetch_pinboard_bookmarks --recent=20

    Fetch the bookmark for one URL:
    ./manage.py fetch_pinboard_bookmarks --url=http://new-aesthetic.tumblr.com/

    Restrict any of the above to one account by adding the account's username:
    ./manage.py fetch_pinboardbookmarks --all --account=philgyford
    """

    singular_noun = "Bookmark"
    plural_noun = "Bookmarks"

    help = "Fetches bookmarks from Pinboard"

    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument(
            "--all", action="store_true", default=False, help="Fetch all bookmarks."
        )
        parser.add_argument(
            "--date",
            action="store",
            default=False,
            help='Fetch bookmarks posted on one day, e.g. "2015-06-20".',
        )
        parser.add_argument(
            "--recent",
            action="store",
            default=False,
            help='Fetch the most recent bookmarks, e.g. "10".',
        )
        parser.add_argument(
            "--url",
            action="store",
            default=False,
            help='Fetch the bookmark for one URL, e.g. "http://www.foo.com".',
        )
        parser.add_argument(
            "--account",
            action="store",
            default=False,
            help="Only fetch for one Pinboard account.",
        )

    def handle(self, *args, **options):

        # We might be fetching for a specific account or all (None).
        account = options["account"] if options["account"] else None

        if options["all"]:
            results = AllBookmarksFetcher().fetch(username=account)

        elif options["date"]:
            results = DateBookmarksFetcher().fetch(
                post_date=options["date"], username=account
            )

        elif options["recent"]:
            results = RecentBookmarksFetcher().fetch(
                num=options["recent"], username=account
            )

        elif options["url"]:
            results = UrlBookmarksFetcher().fetch(url=options["url"], username=account)

        elif options["account"]:
            raise CommandError(
                "Specify --all, --recent, --date= or --url= as well as --account."
            )
        else:
            raise CommandError("Specify --all, --recent, --date= or --url=")

        self.output_results(results, options.get("verbosity", 1))



from django.conf.urls import url

from . import views


app_name = "flickr"

urlpatterns = [
    url(regex=r"^$", view=views.HomeView.as_view(), name="home"),
    url(
        regex=r"^albums/$", view=views.PhotosetListView.as_view(), name="photoset_list"
    ),
    url(regex=r"^tags/$", view=views.TagListView.as_view(), name="tag_list"),
    url(
        regex=r"^tags/(?P<slug>[^/]+)/$",
        view=views.TagDetailView.as_view(),
        name="tag_detail",
    ),
    url(
        regex=r"^(?P<nsid>[\dN@]+)/$",
        view=views.UserDetailView.as_view(),
        name="user_detail",
    ),
    url(
        regex=r"^(?P<nsid>[\dN@]+)/albums/$",
        view=views.UserPhotosetListView.as_view(),
        name="user_photoset_list",
    ),
    url(
        regex=r"^(?P<nsid>[\dN@]+)/albums/(?P<flickr_id>[^/]+)/$",
        view=views.PhotosetDetailView.as_view(),
        name="photoset_detail",
    ),
    url(
        regex=r"^(?P<nsid>[\dN@]+)/tags/(?P<tag_slug>[^/]+)/$",
        view=views.UserTagDetailView.as_view(),
        name="user_tag_detail",
    ),
    url(
        regex=r"^(?P<nsid>[\dN@]+)/(?P<flickr_id>\d+)/$",
        view=views.PhotoDetailView.as_view(),
        name="photo_detail",
    ),
]



from . import FetchCommand
from ...fetch.multifetchers import OriginalFilesMultiAccountFetcher


class Command(FetchCommand):
    """Fetches original photo files from Flickr.

    For all accounts:
        ./manage.py fetch_flickr_originals
        ./manage.py fetch_flickr_originals --all

    For one account:
        ./manage.py fetch_flickr_originals --account=35034346050@N01
        ./manage.py fetch_flickr_originals --account=35034346050@N01 --all
    """

    help = "Fetches the original image files for one or all Flickr Accounts"

    singular_noun = "File"
    plural_noun = "Files"

    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument(
            "--all",
            action="store_true",
            default=False,
            help=(
                "Fetch ALL files, even if they've been downloaded before. "
                "Otherwise, only fetch files that haven't already been downloaded."
            ),
        )

    def handle(self, *args, **options):
        # We might be fetching for a specific account or all (None).
        nsid = options["account"] if options["account"] else None

        results = self.fetch_files(nsid, options["all"])
        self.output_results(results, options.get("verbosity", 1))

    def fetch_files(self, nsid, fetch_all=False):
        return OriginalFilesMultiAccountFetcher(nsid=nsid).fetch(fetch_all=fetch_all)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-26 15:40
from __future__ import unicode_literals
import random

from django.db import migrations, models


def lowercase_slugs(apps, schema_editor):
    """
    Populate the `original_slug` fields with the value of `slug`.
    Then convert all `slug`s to lowercase.

    A bit horrid. In case there are going to be duplicate lowercase slugs,
    we add a random number to the end of the next version.
    """

    Album = apps.get_model("lastfm", "Album")
    slugs = []
    for row in Album.objects.all():
        row.original_slug = row.slug
        row.slug = row.slug.lower()
        if row.slug in slugs:
            row.slug = "%s+%s" % (row.slug, random.randrange(1, 1000000))
        slugs.append(row.slug)
        row.save()

    Artist = apps.get_model("lastfm", "Artist")
    slugs = []
    for row in Artist.objects.all():
        row.original_slug = row.slug
        row.slug = row.slug.lower()
        if row.slug in slugs:
            row.slug = "%s+%s" % (row.slug, random.randrange(1, 1000000))
        slugs.append(row.slug)
        row.save()

    Track = apps.get_model("lastfm", "Track")
    slugs = []
    for row in Track.objects.all():
        row.original_slug = row.slug
        row.slug = row.slug.lower()
        if row.slug in slugs:
            row.slug = "%s+%s" % (row.slug, random.randrange(1, 1000000))
        slugs.append(row.slug)
        row.save()


class Migration(migrations.Migration):

    dependencies = [
        ("lastfm", "0003_auto_20161026_1539"),
    ]

    operations = [
        migrations.RunPython(lowercase_slugs, reverse_code=migrations.RunPython.noop),
        migrations.AlterField(
            model_name="album",
            name="original_slug",
            field=models.TextField(help_text="As used on Last.fm. Mixed case."),
        ),
        migrations.AlterField(
            model_name="artist",
            name="original_slug",
            field=models.TextField(
                help_text="As used on Last.fm. Mixed case.", unique=True
            ),
        ),
        migrations.AlterField(
            model_name="track",
            name="original_slug",
            field=models.TextField(help_text="As used on Last.fm. Mixed case."),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-05 11:56
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0035_auto_20160505_1137"),
    ]

    operations = [
        migrations.RemoveField(model_name="media", name="mp4_bitrate_1",),
        migrations.RemoveField(model_name="media", name="mp4_bitrate_2",),
        migrations.RemoveField(model_name="media", name="mp4_bitrate_3",),
        migrations.RemoveField(model_name="media", name="mp4_url_1",),
        migrations.RemoveField(model_name="media", name="mp4_url_2",),
        migrations.RemoveField(model_name="media", name="mp4_url_3",),
        migrations.RemoveField(model_name="media", name="webm_bitrate",),
        migrations.RemoveField(model_name="media", name="webm_url",),
        migrations.AddField(
            model_name="media",
            name="mp4_url",
            field=models.URLField(
                blank=True, help_text="For Animated GIFs", verbose_name="MP4 URL"
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-24 13:50
from __future__ import unicode_literals

import ditto.flickr.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0016_photo_video_original_file"),
    ]

    operations = [
        migrations.AddField(
            model_name="user",
            name="avatar",
            field=models.ImageField(
                blank=True,
                default="",
                upload_to=ditto.flickr.models.User.avatar_upload_path,
            ),
        ),
        migrations.AlterField(
            model_name="photo",
            name="original_file",
            field=models.ImageField(
                blank=True, default="", upload_to=ditto.flickr.models.Photo.upload_path
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-03 16:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0012_auto_20160503_1457"),
    ]

    operations = [
        migrations.AddField(
            model_name="photoset",
            name="photos_raw",
            field=models.TextField(
                blank=True, help_text="The raw JSON from the API listing the photos."
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0006_auto_20150730_1450"),
    ]

    operations = [
        migrations.RemoveField(model_name="tweet", name="in_reply_to_status_id_str",),
        migrations.RemoveField(model_name="tweet", name="in_reply_to_user_id_str",),
        migrations.RemoveField(model_name="tweet", name="quoted_status_id_str",),
        migrations.RemoveField(model_name="tweet", name="twitter_id_str",),
        migrations.AlterField(
            model_name="tweet",
            name="favorite_count",
            field=models.PositiveIntegerField(
                default=0,
                help_text=b"Approximately how many times this had been favorited when fetched",  # noqa: E501
            ),
        ),
        migrations.AlterField(
            model_name="tweet",
            name="retweet_count",
            field=models.PositiveIntegerField(
                default=0,
                help_text=b"Number of times this had been retweeted when fetched",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-28 15:42
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pinboard", "0021_auto_20160428_1533"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bookmark",
            name="url",
            field=models.TextField(validators=[django.core.validators.URLValidator()]),
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="url_hash",
            field=models.CharField(
                blank=True,
                db_index=True,
                help_text="Slug in the Bookmark's local URL.",
                max_length=12,
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-07-13 11:27
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0048_auto_20160615_1045"),
    ]

    operations = [
        migrations.AlterField(
            model_name="media",
            name="aspect_ratio",
            field=models.CharField(
                blank=True, help_text='eg, "4:3" or "16:9"', max_length=21
            ),
        ),
        migrations.AlterField(
            model_name="media",
            name="dash_url",
            field=models.URLField(
                blank=True, verbose_name="MPEG-DASH URL (.mpd, streaming)"
            ),
        ),
        migrations.AlterField(
            model_name="media",
            name="xmpeg_url",
            field=models.URLField(
                blank=True, verbose_name="X-MPEG URL (HLS, .m3u8, streaming"
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-15 09:11
from __future__ import unicode_literals

import ditto.twitter.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0044_auto_20160613_1600"),
    ]

    operations = [
        migrations.AddField(
            model_name="media",
            name="original_image_file",
            field=models.ImageField(
                blank=True, default="", upload_to=ditto.twitter.models.Media.upload_path
            ),
        ),
    ]



# coding: utf-8
import os

from django.core.management.base import BaseCommand, CommandError

from ...ingest import TweetIngester


class Command(BaseCommand):
    """Imports tweets from a downloaded archive of Tweets.
    Request your archive from https://twitter.com/settings/account

    Usage:
    ./manage.py import_tweets --path=/Users/phil/Downloads/12552_dbeb4be9b8ff5f76d7d486c005cc21c9faa61f66  # noqa: E501
    """

    help = "Imports a complete history of tweets from a downloaded archive"

    def add_arguments(self, parser):
        parser.add_argument(
            "--path",
            action="store",
            default=False,
            help="Path to the directory that is the archive",
        )

    def handle(self, *args, **options):
        # Location of the directory holding the tweet JSON files within the
        # archive:
        subpath = "/data/js/tweets"

        if options["path"]:
            if os.path.isdir(options["path"]):
                tweets_dir = "%s%s" % (options["path"], subpath)
                if os.path.isdir(tweets_dir):
                    result = TweetIngester().ingest(directory=tweets_dir)
                else:
                    raise CommandError(
                        "Expected to find a directory at '%s' containing JSON files"
                        % tweets_dir
                    )
            else:
                raise CommandError("Can't find a directory at '%s'" % options["path"])
        else:
            raise CommandError(
                (
                    "Specify the location of the archive, "
                    "e.g. --path=/Path/To/1234567890_abcdefg12345"
                )
            )

        if options.get("verbosity", 1) > 0:
            if result["success"]:
                tweetnoun = "tweet" if result["tweets"] == 1 else "tweets"
                filenoun = "file" if result["files"] == 1 else "files"

                self.stdout.write(
                    "Imported %s %s from %s %s"
                    % (result["tweets"], tweetnoun, result["files"], filenoun)
                )
            else:

                self.stderr.write(
                    "Failed to import tweets: %s" % (result["messages"][0])
                )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0007_auto_20150807_1432"),
    ]

    operations = [
        migrations.RemoveField(model_name="user", name="twitter_id_str",),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0026_auto_20151110_1131"),
    ]

    operations = [
        migrations.RemoveField(model_name="user", name="profile_image_url",),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-15 10:38
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0045_media_original_image_file"),
    ]

    operations = [
        migrations.RenameField(
            model_name="media", old_name="original_image_file", new_name="image_file",
        ),
        migrations.AlterField(
            model_name="media",
            name="image_url",
            field=models.URLField(help_text="URL of the image itself on Twitter.com"),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-29 15:59
from __future__ import unicode_literals

import ditto.core.models
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0007_auto_20160414_1637"),
    ]

    operations = [
        migrations.CreateModel(
            name="Photoset",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "time_created",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="The time this item was created in the database.",
                    ),
                ),
                (
                    "time_modified",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="The time this item was last saved to the database.",
                    ),
                ),
                ("flickr_id", models.BigIntegerField(unique=True)),
                ("title", models.CharField(max_length=255)),
                (
                    "description",
                    models.TextField(blank=True, help_text="Can contain HTML"),
                ),
                ("photo_count", models.PositiveIntegerField(default=0)),
                ("video_count", models.PositiveIntegerField(default=0)),
                (
                    "view_count",
                    models.PositiveIntegerField(
                        default=0,
                        help_text="How many times this had been viewed when fetched",
                    ),
                ),
                (
                    "comment_count",
                    models.PositiveIntegerField(
                        default=0, help_text="How many comments this had when fetched"
                    ),
                ),
                (
                    "last_update_time",
                    models.DateTimeField(
                        blank=True,
                        help_text="The last time the set was modified on Flickr. UTC.",
                        null=True,
                    ),
                ),
                (
                    "flickr_created_time",
                    models.DateTimeField(
                        blank=True,
                        help_text="When the set was created on Flickr. UTC.",
                        null=True,
                    ),
                ),
                (
                    "raw",
                    models.TextField(
                        blank=True, help_text="The raw JSON from the API."
                    ),
                ),
            ],
            options={"ordering": ["-flickr_created_time"]},
            bases=(ditto.core.models.DiffModelMixin, models.Model),
        ),
        migrations.AlterField(
            model_name="photo",
            name="comment_count",
            field=models.PositiveIntegerField(
                default=0, help_text="How many comments this had when fetched"
            ),
        ),
        migrations.AlterField(
            model_name="photo",
            name="sizes_raw",
            field=models.TextField(
                blank=True,
                help_text="The raw JSON from the API - flickr.photos.getSizes.",
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="raw",
            field=models.TextField(blank=True, help_text="The raw JSON from the API."),
        ),
        migrations.AddField(
            model_name="photoset",
            name="primary_photo",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="flickr.Photo",
            ),
        ),
        migrations.AddField(
            model_name="photoset",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="flickr.User"
            ),
        ),
    ]



# coding: utf-8
from ....core.management.commands import DittoBaseCommand
from ...fetch.fetchers import FilesFetcher


class Command(DittoBaseCommand):
    """Fetches images and Animated GIFs' video files from Twitter.

    eg:
    ./manage.py fetch_twitter_files
    ./manage.py fetch_twitter_files --all

    """

    help = "Fetches images and Animated GIFs' video files from Twitter"

    singular_noun = "File"
    plural_noun = "Files"

    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument(
            "--all",
            action="store_true",
            default=False,
            help=(
                "Fetch ALL files, even if they've been downloaded before. "
                "Otherwise, only fetch files that haven't already been downloaded."
            )
        )

    def handle(self, *args, **options):
        results = FilesFetcher().fetch(fetch_all=options["all"])
        self.output_results(results, options.get("verbosity", 1))



from django.conf.urls import url

from . import views


app_name = "lastfm"


# The pattern for matching an Album/Artist/Track slug:
slug_chars = "[\w.,:;=@&+%()$!°’~-]+"  # noqa: W605


urlpatterns = [
    url(regex=r"^$", view=views.HomeView.as_view(), name="home"),
    url(
        regex=r"^library/$", view=views.ScrobbleListView.as_view(), name="scrobble_list"
    ),
    url(
        regex=r"^library/albums/$",
        view=views.AlbumListView.as_view(),
        name="album_list",
    ),
    url(
        regex=r"^library/artists/$",
        view=views.ArtistListView.as_view(),
        name="artist_list",
    ),
    url(
        regex=r"^library/tracks/$",
        view=views.TrackListView.as_view(),
        name="track_list",
    ),
    url(
        regex=r"^music/(?P<artist_slug>%s)/$" % slug_chars,
        view=views.ArtistDetailView.as_view(),
        name="artist_detail",
    ),
    url(
        regex=r"^music/(?P<artist_slug>%s)/\+albums/$" % slug_chars,
        view=views.ArtistAlbumsView.as_view(),
        name="artist_albums",
    ),
    url(
        regex=r"^music/(?P<artist_slug>%s)/(?P<album_slug>%s)/$"
        % (slug_chars, slug_chars),
        view=views.AlbumDetailView.as_view(),
        name="album_detail",
    ),
    url(
        regex=r"^music/(?P<artist_slug>%s)/_/(?P<track_slug>%s)/$"
        % (slug_chars, slug_chars),
        view=views.TrackDetailView.as_view(),
        name="track_detail",
    ),
    # User pages.
    url(
        regex=r"^user/(?P<username>[a-z0-9]+)/$",
        view=views.UserDetailView.as_view(),
        name="user_detail",
    ),
    url(
        regex=r"^user/(?P<username>[a-z0-9]+)/library/$",
        view=views.UserScrobbleListView.as_view(),
        name="user_scrobble_list",
    ),
    url(
        regex=r"^user/(?P<username>[a-z0-9]+)/library/albums/$",
        view=views.UserAlbumListView.as_view(),
        name="user_album_list",
    ),
    url(
        regex=r"^user/(?P<username>[a-z0-9]+)/library/artists/$",
        view=views.UserArtistListView.as_view(),
        name="user_artist_list",
    ),
    url(
        regex=r"^user/(?P<username>[a-z0-9]+)/library/tracks/$",
        view=views.UserTrackListView.as_view(),
        name="user_track_list",
    ),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0012_auto_20150814_1730"),
    ]

    operations = [
        migrations.AddField(
            model_name="user",
            name="favorites",
            field=models.ManyToManyField(
                related_name="favoriting_users", to="twitter.Tweet"
            ),
        ),
    ]



from django.contrib import admin
from django.db import models
from django.forms import Textarea, TextInput

from taggit.managers import TaggableManager
from taggit.forms import TagWidget

from ..core.admin import DittoItemModelAdmin
from .models import Account, Bookmark


@admin.register(Account)
class AccountAdmin(admin.ModelAdmin):
    list_display = (
        "username",
        "is_active",
        "time_created",
        "time_modified",
    )

    fieldsets = (
        (None, {"fields": ("username", "url", "is_active",)}),
        (
            "API",
            {
                "fields": ("api_token",),
                "description": (
                    "Your API Token can be found at "
                    '<a href="https://pinboard.in/settings/password">'
                    "pinboard.in/settings/password</a>"
                ),
            },
        ),
        ("Data", {"fields": ("time_created", "time_modified",)}),
    )
    readonly_fields = (
        "time_created",
        "time_modified",
    )


@admin.register(Bookmark)
class BookmarkAdmin(DittoItemModelAdmin):
    list_display = (
        "title",
        "post_time",
        "account",
    )
    list_filter = (
        "post_time",
        "is_private",
        "to_read",
        "account",
    )

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "account",
                    "title",
                    "url",
                    "description",
                    "summary",
                    "tags",
                    "post_time",
                    "post_year_str",
                    "url_hash",
                )
            },
        ),
        ("Options", {"fields": ("is_private", "to_read",)}),
        ("Data", {"fields": ("raw", "fetch_time", "time_created", "time_modified",)}),
    )

    formfield_overrides = {
        # Make the inputs full-width.
        models.CharField: {"widget": TextInput(attrs={"class": "vLargeTextField"})},
        # Reduce the number of rows; similar to Pinboard's description field.
        models.TextField: {
            "widget": Textarea(
                attrs={"class": "vLargeTextField", "cols": 40, "rows": 4}
            )
        },
        # Make the input full-width.
        TaggableManager: {"widget": TagWidget(attrs={"class": "vLargeTextField"})},
    }

    readonly_fields = (
        "post_year_str",
        "raw",
        "fetch_time",
        "time_created",
        "time_modified",
    )
    search_fields = (
        "title",
        "url",
        "description",
    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0003_auto_20150730_1112"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="favorites_count",
            field=models.PositiveIntegerField(
                default=0,
                help_text=b"The number of tweets this user has favorited in the account\xe2\x80\x99s lifetime",  # noqa: E501
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="followers_count",
            field=models.PositiveIntegerField(
                default=0, help_text=b"The number of followers this account has"
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="friends_count",
            field=models.PositiveIntegerField(
                default=0, help_text=b"Tne number of users this account is following."
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="listed_count",
            field=models.PositiveIntegerField(
                default=0,
                help_text=b"The number of public lists this user is a member of",
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="statuses_count",
            field=models.PositiveIntegerField(
                default=0,
                help_text=b"The number of tweets, including retweets, by this user",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0028_auto_20151110_1139"),
    ]

    # Manually re-jigged to avoid losing any data:
    operations = [
        migrations.AlterField(
            model_name="tweet",
            name="created_at",
            field=models.DateTimeField(
                help_text="The time the item was originally posted/created on its service.",  # noqa: E501
                blank=True,
                null=True,
            ),
        ),
        migrations.RenameField(
            model_name="tweet", old_name="created_at", new_name="post_time",
        ),
        migrations.AlterModelOptions(
            name="tweet", options={"ordering": ["-post_time"]},
        ),
    ]



from django.contrib import admin

from ..core.admin import DittoItemModelAdmin
from .models import Account, Album, Artist, Scrobble, Track


@admin.register(Account)
class AccountAdmin(admin.ModelAdmin):
    list_display = (
        "username",
        "realname",
        "is_active",
        "time_created",
    )
    search_fields = (
        "name",
        "artist",
        "mbid",
    )

    fieldsets = (
        (None, {"fields": ("username", "realname", "api_key", "is_active",)}),
        ("Data", {"fields": ("time_created", "time_modified",)}),
    )

    readonly_fields = (
        "time_created",
        "time_modified",
    )


@admin.register(Artist)
class ArtistAdmin(admin.ModelAdmin):
    list_display = (
        "name",
        "mbid",
        "time_created",
    )
    search_fields = (
        "name",
        "mbid",
    )

    fieldsets = (
        (None, {"fields": ("name", "slug", "original_slug", "mbid",)}),
        ("Data", {"fields": ("time_created", "time_modified",)}),
    )

    readonly_fields = (
        "time_created",
        "time_modified",
    )


@admin.register(Track)
class TrackAdmin(admin.ModelAdmin):
    list_display = (
        "name",
        "artist",
        "mbid",
        "time_created",
    )
    search_fields = (
        "name",
        "artist__name",
        "mbid",
    )

    fieldsets = (
        (None, {"fields": ("name", "slug", "original_slug", "artist", "mbid",)}),
        ("Data", {"fields": ("time_created", "time_modified",)}),
    )

    readonly_fields = (
        "time_created",
        "time_modified",
    )
    raw_id_fields = ("artist",)


@admin.register(Album)
class AlbumAdmin(admin.ModelAdmin):
    list_display = (
        "name",
        "artist",
        "mbid",
        "time_created",
    )
    search_fields = (
        "name",
        "artist__name",
    )

    fieldsets = (
        (None, {"fields": ("name", "slug", "original_slug", "artist", "mbid",)}),
        ("Data", {"fields": ("time_created", "time_modified",)}),
    )

    readonly_fields = (
        "time_created",
        "time_modified",
    )
    raw_id_fields = ("artist",)


@admin.register(Scrobble)
class ScrobbleAdmin(DittoItemModelAdmin):
    list_display = (
        "title",
        "post_time",
    )
    list_filter = ("post_time",)
    search_fields = (
        "title",
        "track__name",
    )

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "account",
                    "artist",
                    "track",
                    "album",
                    "post_time",
                    "post_year_str",
                )
            },
        ),
        ("Data", {"fields": ("raw", "fetch_time", "time_created", "time_modified",)}),
    )

    raw_id_fields = (
        "artist",
        "track",
        "album",
    )
    readonly_fields = (
        "post_year_str",
        "raw",
        "fetch_time",
        "time_created",
        "time_modified",
    )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("pinboard", "0006_auto_20150723_1322"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="account", options={"ordering": ["username"]},
        ),
        migrations.AlterModelOptions(
            name="bookmark", options={"ordering": ["-post_time"]},
        ),
        migrations.AlterField(
            model_name="account",
            name="time_created",
            field=models.DateTimeField(
                help_text=b"The time this item was created in the database.",
                auto_now_add=True,
            ),
        ),
        migrations.AlterField(
            model_name="account",
            name="time_modified",
            field=models.DateTimeField(
                help_text=b"The time this item was last saved to the database.",
                auto_now=True,
            ),
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="fetch_time",
            field=models.DateTimeField(
                help_text=b"The time the item's data was last fetched, and was new or changed.",  # noqa: E501
                null=True,
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="is_private",
            field=models.BooleanField(
                default=False,
                help_text=b"If set, this item will not be shown on public-facing pages.",  # noqa: E501
            ),
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="time_created",
            field=models.DateTimeField(
                help_text=b"The time this item was created in the database.",
                auto_now_add=True,
            ),
        ),
        migrations.AlterField(
            model_name="bookmark",
            name="time_modified",
            field=models.DateTimeField(
                help_text=b"The time this item was last saved to the database.",
                auto_now=True,
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0017_account_is_active"),
    ]

    operations = [
        migrations.AddField(
            model_name="tweet",
            name="text_html",
            field=models.TextField(
                blank=True, help_text="An HTMLified version of the Tweet's text"
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0005_auto_20150730_1350"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="account", options={"ordering": ["-time_created"]},
        ),
        migrations.RemoveField(model_name="account", name="screen_name",),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-03 14:38
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0042_auto_20160603_1422"),
    ]

    operations = [
        migrations.RemoveField(model_name="media", name="is_private",),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion
import ditto.core.models
import taggit.managers


class Migration(migrations.Migration):

    dependencies = [
        ("taggit", "0002_auto_20150616_2121"),
    ]

    operations = [
        migrations.CreateModel(
            name="Account",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        primary_key=True,
                        auto_created=True,
                    ),
                ),
                (
                    "time_created",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="The time this item was created in the database.",
                    ),
                ),
                (
                    "time_modified",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="The time this item was last saved to the database.",
                    ),
                ),
                (
                    "api_key",
                    models.CharField(
                        max_length=255, blank=True, verbose_name="API Key"
                    ),
                ),
                (
                    "api_secret",
                    models.CharField(
                        max_length=255, blank=True, verbose_name="API Secret"
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(
                        default=True, help_text="If false, new Photos won't be fetched."
                    ),
                ),
            ],
            options={"ordering": ["user__realname"]},
        ),
        migrations.CreateModel(
            name="Photo",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        primary_key=True,
                        auto_created=True,
                    ),
                ),
                (
                    "time_created",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="The time this item was created in the database.",
                    ),
                ),
                (
                    "time_modified",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="The time this item was last saved to the database.",
                    ),
                ),
                ("title", models.CharField(max_length=255, blank=True)),
                (
                    "permalink",
                    models.URLField(
                        blank=True,
                        help_text="URL of the item on the service's website.",
                    ),
                ),
                (
                    "summary",
                    models.CharField(
                        max_length=255,
                        blank=True,
                        help_text="eg, Initial text of a blog post, start of the description of a photo, all of a Tweet's text, etc. No HTML.",  # noqa: E501
                    ),
                ),
                (
                    "is_private",
                    models.BooleanField(
                        default=False,
                        help_text="If true, this item will not be shown on public-facing pages.",  # noqa: E501
                    ),
                ),
                (
                    "fetch_time",
                    models.DateTimeField(
                        null=True,
                        blank=True,
                        help_text="The time the item's data was last fetched, and was new or changed.",  # noqa: E501
                    ),
                ),
                (
                    "post_time",
                    models.DateTimeField(
                        null=True,
                        blank=True,
                        help_text="The time the item was originally posted/created on its service.",  # noqa: E501
                    ),
                ),
                (
                    "latitude",
                    models.DecimalField(
                        null=True, blank=True, decimal_places=9, max_digits=12
                    ),
                ),
                (
                    "longitude",
                    models.DecimalField(
                        null=True, blank=True, decimal_places=9, max_digits=12
                    ),
                ),
                (
                    "raw",
                    models.TextField(
                        blank=True, help_text="eg, the raw JSON from the API."
                    ),
                ),
                (
                    "flickr_id",
                    models.BigIntegerField(
                        help_text="ID of this photo on Flickr.",
                        db_index=True,
                        unique=True,
                    ),
                ),
                (
                    "description",
                    models.TextField(blank=True, help_text="Can contain HTML"),
                ),
                ("secret", models.CharField(max_length=20)),
                ("original_secret", models.CharField(max_length=20)),
                ("server", models.CharField(max_length=20)),
                ("farm", models.PositiveSmallIntegerField()),
                (
                    "license",
                    models.CharField(
                        max_length=50,
                        choices=[
                            ("0", "All Rights Reserved"),
                            ("1", "Attribution-NonCommercial-ShareAlike License"),
                            ("2", "Attribution-NonCommercial License"),
                            ("3", "Attribution-NonCommercial-NoDerivs License"),
                            ("4", "Attribution License"),
                            ("5", "Attribution-ShareAlike License"),
                            ("6", "Attribution-NoDerivs License"),
                            ("7", "No known copyright restrictions"),
                            ("8", "United States Government Work"),
                        ],
                    ),
                ),
                (
                    "rotation",
                    models.PositiveSmallIntegerField(
                        help_text="Current clockwise rotation, in degrees, by which the smaller image sizes differ from the original image.",  # noqa: E501
                        default=0,
                    ),
                ),
                (
                    "original_format",
                    models.CharField(max_length=10, help_text="eg, 'png'"),
                ),
                (
                    "safety_level",
                    models.PositiveSmallIntegerField(
                        choices=[
                            (0, "none"),
                            (1, "Safe"),
                            (2, "Moderate"),
                            (3, "Restricted"),
                        ],
                        default=0,
                    ),
                ),
                (
                    "has_people",
                    models.BooleanField(
                        default=False, help_text="Are there Flickr users in this photo?"
                    ),
                ),
                (
                    "last_update_time",
                    models.DateTimeField(
                        null=True,
                        blank=True,
                        help_text="The last time the photo, or any of its metadata (tags, comments, etc.) was modified on Flickr. UTC.",  # noqa: E501
                    ),
                ),
                (
                    "taken_time",
                    models.DateTimeField(
                        null=True,
                        blank=True,
                        help_text="In the Flickr user's timezone.",
                    ),
                ),
                (
                    "taken_granularity",
                    models.PositiveSmallIntegerField(
                        choices=[
                            (0, "Y-m-d H:i:s"),
                            (4, "Y-m"),
                            (6, "Y"),
                            (8, "Circa..."),
                        ],
                        default=0,
                    ),
                ),
                ("taken_unknown", models.BooleanField(default=False)),
                (
                    "view_count",
                    models.PositiveIntegerField(
                        help_text="How many times this had been viewed when fetched",
                        default=0,
                    ),
                ),
                (
                    "comment_count",
                    models.PositiveIntegerField(
                        help_text="How many comments this had been when fetched",
                        default=0,
                    ),
                ),
                (
                    "media",
                    models.CharField(
                        max_length=10,
                        choices=[("photo", "Photo"), ("video", "Video")],
                        default="photo",
                    ),
                ),
                (
                    "sizes_raw",
                    models.TextField(
                        blank=True,
                        help_text="eg, the raw JSON from the API - flickr.photos.getSizes.",  # noqa: E501
                    ),
                ),
                (
                    "width_t",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Thumbnail width"
                    ),
                ),
                (
                    "height_t",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Thumbnail height"
                    ),
                ),
                (
                    "width_m",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Small width"
                    ),
                ),
                (
                    "height_m",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Small height"
                    ),
                ),
                (
                    "width_n",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Small 320 width"
                    ),
                ),
                (
                    "height_n",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Small 320 height"
                    ),
                ),
                (
                    "width",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Medium width"
                    ),
                ),
                (
                    "height",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Medium height"
                    ),
                ),
                (
                    "width_z",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Medium 640 width"
                    ),
                ),
                (
                    "height_z",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Medium 640 height"
                    ),
                ),
                (
                    "width_c",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Medium 800 width"
                    ),
                ),
                (
                    "height_c",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Medium 800 height"
                    ),
                ),
                (
                    "width_b",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Large width"
                    ),
                ),
                (
                    "height_b",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Large height"
                    ),
                ),
                (
                    "width_h",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Large 1600 width"
                    ),
                ),
                (
                    "height_h",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Large 1600 height"
                    ),
                ),
                (
                    "width_k",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Large 2048 width"
                    ),
                ),
                (
                    "height_k",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Large 2048 height"
                    ),
                ),
                (
                    "width_o",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Original width"
                    ),
                ),
                (
                    "height_o",
                    models.PositiveSmallIntegerField(
                        null=True, blank=True, verbose_name="Original height"
                    ),
                ),
                (
                    "geo_is_private",
                    models.BooleanField(
                        default=False,
                        help_text="If true, the Photo's location info should not be displayed.",  # noqa: E501
                    ),
                ),
                (
                    "location_accuracy",
                    models.PositiveSmallIntegerField(
                        null=True,
                        blank=True,
                        default=1,
                        help_text="World is 1; Country is ~3; Region is ~6; City is ~11; Street is ~16.",  # noqa: E501
                    ),
                ),
                (
                    "location_context",
                    models.PositiveSmallIntegerField(
                        choices=[(0, "not defined"), (1, "indoors"), (2, "outdoors")],
                        default=0,
                    ),
                ),
                ("location_place_id", models.CharField(max_length=30, blank=True)),
                ("location_woeid", models.CharField(max_length=30, blank=True)),
                ("locality_name", models.CharField(max_length=255, blank=True)),
                ("locality_place_id", models.CharField(max_length=30, blank=True)),
                ("locality_woeid", models.CharField(max_length=30, blank=True)),
                ("county_name", models.CharField(max_length=255, blank=True)),
                ("county_place_id", models.CharField(max_length=30, blank=True)),
                ("county_woeid", models.CharField(max_length=30, blank=True)),
                ("region_name", models.CharField(max_length=255, blank=True)),
                ("region_place_id", models.CharField(max_length=30, blank=True)),
                ("region_woeid", models.CharField(max_length=30, blank=True)),
                ("country_name", models.CharField(max_length=255, blank=True)),
                ("country_place_id", models.CharField(max_length=30, blank=True)),
                ("country_woeid", models.CharField(max_length=30, blank=True)),
                (
                    "exif_raw",
                    models.TextField(
                        blank=True,
                        help_text="The raw JSON from the API from flickr.photos.getExif.",  # noqa: E501
                    ),
                ),
                ("exif_camera", models.CharField(max_length=50, blank=True)),
                (
                    "exif_lens_model",
                    models.CharField(
                        max_length=50,
                        blank=True,
                        help_text="eg, 'E PZ 16-50mm F3.5-5.6 OSS'.",
                    ),
                ),
                (
                    "exif_aperture",
                    models.CharField(
                        max_length=30, blank=True, help_text="eg, 'f/13.0'."
                    ),
                ),
                (
                    "exif_exposure",
                    models.CharField(
                        max_length=30, blank=True, help_text="eg, '0.01 sec (1/100)'."
                    ),
                ),
                (
                    "exif_flash",
                    models.CharField(
                        max_length=30, blank=True, help_text="eg, 'Off, Did not fire'."
                    ),
                ),
                (
                    "exif_focal_length",
                    models.CharField(
                        max_length=10, blank=True, help_text="eg, '38 mm.'"
                    ),
                ),
                ("exif_iso", models.IntegerField(blank=True, help_text="eg, '100'.")),
            ],
            options={"ordering": ("-taken_time",)},
            bases=(ditto.core.models.DiffModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="TaggedPhoto",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        primary_key=True,
                        auto_created=True,
                    ),
                ),
                (
                    "flickr_id",
                    models.CharField(
                        max_length=200,
                        help_text="The tag's ID on Flickr",
                        verbose_name="Flickr ID",
                    ),
                ),
                ("machine_tag", models.BooleanField(default=False)),
            ],
            options={"verbose_name": "Photo/Tag Relationship"},
        ),
        migrations.CreateModel(
            name="User",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        primary_key=True,
                        auto_created=True,
                    ),
                ),
                (
                    "time_created",
                    models.DateTimeField(
                        auto_now_add=True,
                        help_text="The time this item was created in the database.",
                    ),
                ),
                (
                    "time_modified",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="The time this item was last saved to the database.",
                    ),
                ),
                (
                    "nsid",
                    models.CharField(max_length=50, verbose_name="NSID", unique=True),
                ),
                ("is_pro", models.BooleanField(verbose_name="Is Pro?", default=False)),
                ("iconserver", models.CharField(max_length=20)),
                ("iconfarm", models.PositiveIntegerField()),
                ("username", models.CharField(max_length=50, unique=True)),
                ("realname", models.CharField(max_length=255)),
                ("location", models.CharField(max_length=255, blank=True)),
                (
                    "description",
                    models.TextField(blank=True, help_text="May contain HTML"),
                ),
                (
                    "photos_url",
                    models.URLField(max_length=255, verbose_name="Photos URL"),
                ),
                (
                    "profile_url",
                    models.URLField(max_length=255, verbose_name="Profile URL"),
                ),
                ("photos_count", models.PositiveIntegerField(default=0)),
                ("photos_views", models.PositiveIntegerField(default=0)),
                ("photos_first_date", models.DateTimeField(null=True)),
                ("photos_first_date_taken", models.DateTimeField(null=True)),
                (
                    "fetch_time",
                    models.DateTimeField(
                        null=True,
                        blank=True,
                        help_text="The time the data was last fetched, and was new or changed.",  # noqa: E501
                    ),
                ),
                (
                    "raw",
                    models.TextField(
                        blank=True, help_text="eg, the raw JSON from the API."
                    ),
                ),
                (
                    "timezone_id",
                    models.CharField(max_length=50, help_text="eg, 'Europe/London'."),
                ),
            ],
            options={"ordering": ["realname"]},
            bases=(ditto.core.models.DiffModelMixin, models.Model),
        ),
        migrations.AddField(
            model_name="taggedphoto",
            name="author",
            field=models.ForeignKey(to="flickr.User", on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name="taggedphoto",
            name="content_object",
            field=models.ForeignKey(
                to="flickr.Photo",
                related_name="flickr_taggedphoto_items",
                on_delete=models.CASCADE,
            ),
        ),
        migrations.AddField(
            model_name="taggedphoto",
            name="tag",
            field=models.ForeignKey(
                to="taggit.Tag",
                related_name="flickr_taggedphoto_items",
                on_delete=models.CASCADE,
            ),
        ),
        migrations.AddField(
            model_name="photo",
            name="tags",
            field=taggit.managers.TaggableManager(
                blank=True,
                to="taggit.Tag",
                verbose_name="Tags",
                help_text="A comma-separated list of tags.",
                through="flickr.TaggedPhoto",
            ),
        ),
        migrations.AddField(
            model_name="photo",
            name="user",
            field=models.ForeignKey(to="flickr.User", on_delete=models.CASCADE),
        ),
        migrations.AddField(
            model_name="account",
            name="user",
            field=models.ForeignKey(
                null=True,
                blank=True,
                to="flickr.User",
                on_delete=django.db.models.deletion.SET_NULL,
            ),
        ),
    ]



from django.conf.urls import url

from . import views


app_name = "ditto"

urlpatterns = [
    url(regex=r"^$", view=views.HomeView.as_view(), name="home"),
    # url(
    # regex=r"^tags$",
    # view=views.TagListView.as_view(),
    # name='tag_list'
    # ),
    # url(
    # regex=r"^tags/(?P<slug>[^/]+)$",
    # view=views.TagDetailView.as_view(),
    # name='tag_detail'
    # ),
    url(
        # /2016/04/18/twitter/favorites
        regex=r"^(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})(?:/(?P<app>[a-z]+))?(?:/(?P<variety>[a-z\/]+|))?$",  # noqa: E501
        view=views.DayArchiveView.as_view(),
        name="day_archive",
    ),
]



import codecs
import os
import re
import sys
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), "README.rst")) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


def read(filepath):
    return codecs.open(filepath, "r", "utf-8").read()


def get_entity(package, entity):
    """
    eg, get_entity('ditto', 'version') returns `__version__` value in
    `__init__.py`.
    """
    init_py = open(os.path.join(package, "__init__.py")).read()
    find = "__%s__ = ['\"]([^'\"]+)['\"]" % entity
    return re.search(find, init_py).group(1)


def get_version():
    return get_entity("ditto", "version")


def get_license():
    return get_entity("ditto", "license")


def get_author():
    return get_entity("ditto", "author")


def get_author_email():
    return get_entity("ditto", "author_email")


# Do `python setup.py tag` to tag with the current version number.
if sys.argv[-1] == "tag":
    os.system("git tag -a %s -m 'version %s'" % (get_version(), get_version()))
    os.system("git push --tags")
    sys.exit()

# Do `python setup.py publish` to send current version to PyPI.
if sys.argv[-1] == "publish":
    os.system("python setup.py sdist")
    os.system(
        "twine upload --config-file=.pypirc dist/django-ditto-%s.tar.gz"
        % (get_version())
    )
    sys.exit()

# Do `python setup.py testpublish` to send current version to Test PyPI.
# OUT OF DATE
if sys.argv[-1] == "testpublish":
    os.system("python setup.py sdist")
    os.system(
        "twine upload --config-file=.pypirc --repository-url https://test.pypi.org/legacy/ dist/django-ditto-%s.tar.gz"  # noqa: E501
        % (get_version())
    )
    # os.system("python setup.py bdist_wheel upload")
    sys.exit()

dev_require = ["django-debug-toolbar>=2.0,<3.0", "flake8>=3.8,<3.9", "black==19.10b0"]
tests_require = dev_require + [
    "factory-boy>=2.12.0,<3.0",
    "freezegun>=0.3.12,<0.4",
    "responses>=0.10.7,<1.0",
    "coverage",
]

setup(
    name="django-ditto",
    version=get_version(),
    packages=["ditto"],
    install_requires=[
        "django-imagekit>=4.0,<4.1",
        "django-sortedm2m>=3.0.0,<3.1",
        "django-taggit>=1.2.0,<1.3",
        "flickrapi>=2.4,<2.5",
        "pillow>=6.1.0,<7.2",
        "pytz",
        "twitter-text-python>=1.1.1,<1.2",
        "twython>=3.7.0,<3.9",
    ],
    dependency_links=[],
    tests_require=tests_require,
    extras_require={"dev": dev_require + ["Django>=3.0,<3.1"], "test": tests_require},
    include_package_data=True,
    license=get_license(),
    description=(
        "A Django app to copy stuff from your accounts on "
        "Flickr, Last.fm, Pinboard and Twitter."
    ),
    long_description=read(os.path.join(os.path.dirname(__file__), "README.rst")),
    url="https://github.com/philgyford/django-ditto",
    author=get_author(),
    author_email=get_author_email(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    keywords="ditto twitter flickr pinboard last.fm",
    project_urls={
        "Blog posts": "https://www.gyford.com/phil/writing/tags/django-ditto/",
        "Bug Reports": "https://github.com/philgyford/django-ditto/issues",
        "Documentation": "https://django-ditto.readthedocs.io/",
        "Source": "https://github.com/philgyford/django-ditto",
    },
)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-07-13 14:44
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0050_auto_20160713_1400"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="url",
            field=models.URLField(
                blank=True,
                default="",
                help_text="A URL provided by the user as part of their profile",
                max_length=255,
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-02 16:45
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import sortedm2m.fields


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0010_photo_photosets"),
    ]

    operations = [
        migrations.RemoveField(model_name="photo", name="photosets",),
        migrations.AddField(
            model_name="photoset",
            name="photos",
            field=sortedm2m.fields.SortedManyToManyField(
                help_text=None, related_name="photosets", to="flickr.Photo"
            ),
        ),
        migrations.AlterField(
            model_name="photoset",
            name="primary_photo",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="primary_photosets",
                to="flickr.Photo",
            ),
        ),
    ]



import datetime
import factory

from . import models
from ..core.utils import datetime_now


class AccountFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Account

    username = factory.Sequence(lambda n: "user%d" % n)
    url = factory.LazyAttribute(lambda obj: "https://pinboard.com/%s" % obj.username)
    api_token = factory.LazyAttribute(lambda obj: "%s:123ABC" % obj.username)


class BookmarkFactory(factory.DjangoModelFactory):
    class Meta:
        model = models.Bookmark

    # DittoItem properties:
    title = factory.Sequence(lambda n: "A Title %d" % n)
    is_private = False

    # Bookmark properties:
    account = factory.SubFactory(AccountFactory)
    url = factory.Sequence(lambda n: "http://www.example.com/%d" % n)

    post_time = factory.LazyAttribute(
        lambda o: datetime_now() - datetime.timedelta(hours=1)
    )
    description = factory.Sequence(lambda n: "A description of %d" % n)
    to_read = False

    # Leave tags empty by default.



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("pinboard", "0015_auto_20151110_1308"),
    ]

    operations = [
        migrations.AlterField(
            model_name="account",
            name="api_token",
            field=models.CharField(
                verbose_name="API Token",
                max_length=51,
                help_text='eg, "philgyford:1234567890ABCDEFGHIJ"',
            ),
        ),
        migrations.AlterField(
            model_name="account",
            name="url",
            field=models.URLField(
                verbose_name="URL",
                unique=True,
                max_length=255,
                help_text="eg, 'https://pinboard.in/u:philgyford'",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0021_video"),
    ]

    operations = [
        migrations.RemoveField(model_name="photo", name="tweet",),
        migrations.RemoveField(model_name="video", name="tweet",),
        migrations.RemoveField(model_name="tweet", name="photos_count",),
        migrations.AddField(
            model_name="tweet",
            name="media_count",
            field=models.PositiveSmallIntegerField(
                default=0,
                blank=True,
                help_text="Number of Photos/Videos attached to this Tweet",
            ),
        ),
        migrations.DeleteModel(name="Photo",),
        migrations.DeleteModel(name="Video",),
    ]



# Generated by Django 2.0 on 2018-01-04 14:57

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("lastfm", "0007_set_post_year"),
    ]

    operations = [
        migrations.AlterField(
            model_name="scrobble",
            name="album",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="scrobbles",
                to="lastfm.Album",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0033_auto_20160421_1602"),
    ]

    operations = [
        migrations.AddField(
            model_name="user",
            name="description_html",
            field=models.TextField(
                help_text="An HTMLified version of the User's description", blank=True
            ),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ("twitter", "0030_auto_20151119_1649"),
    ]

    operations = [
        migrations.AlterField(
            model_name="tweet",
            name="fetch_time",
            field=models.DateTimeField(
                blank=True,
                help_text="The time the item's data was last fetched.",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="fetch_time",
            field=models.DateTimeField(
                blank=True, help_text="The time the data was last fetched.", null=True
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-11-17 16:57
from __future__ import unicode_literals

from django.db import migrations


def set_post_year(apps, schema_editor):
    """
    Re-saves every scrobble, so that its auto-generated `post_year` can
    be set.
    """
    Scrobble = apps.get_model("lastfm", "Scrobble")
    for row in Scrobble.objects.all():
        row.post_year = row.post_time.year
        row.save()


class Migration(migrations.Migration):

    dependencies = [
        ("lastfm", "0006_scrobble_post_year"),
    ]

    operations = [
        migrations.RunPython(set_post_year, reverse_code=migrations.RunPython.noop),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-07-13 11:27
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("flickr", "0018_auto_20160525_1011"),
    ]

    operations = [
        migrations.AlterField(
            model_name="photo",
            name="safety_level",
            field=models.PositiveSmallIntegerField(
                choices=[(0, "None"), (1, "Safe"), (2, "Moderate"), (3, "Restricted")],
                default=0,
            ),
        ),
    ]
